#!/usr/bin/env python
# coding: utf-8
from ngsolve import *
import numpy as np
import sys
import os
#
import subprocess
#
from netgen.read_gmsh import ReadGmsh
# initialise mpi
comm = mpi_world
rank = comm.rank
nproc= comm.size
PI = np.pi;

model = sys.argv[1]

waveno = 6.0*PI
polOrder = 2
elmperlam = 4 
wavelength = 2.0*PI/waveno
helem = wavelength / elmperlam
gmshmeshfilename = 'fullmodel.msh'
if (model=='full'):
    volmeshfilename = 'fullmodel.vol'
    
if (model=='half'): 
    volmeshfilename = 'halfmodel.vol'

ngmesh = netgen.meshing.Mesh(dim=3, comm=comm)
ngmesh.Load(volmeshfilename)  
mesh = Mesh(ngmesh)   

mesh.SetPML(pml.Radial(origin=(0,0,0), rad=1.6, alpha=1j), definedon="air")

fes = H1(mesh, complex=True, order=polOrder)
u = fes.TrialFunction()
v = fes.TestFunction()

g = 1;
beta = 1j*waveno
a = BilinearForm (fes, symmetric=True)
a += SymbolicBFI (grad(u)*grad(v) )
a += SymbolicBFI (-waveno*waveno*u*v)
#a += SymbolicBFI(beta*u*v, definedon=mesh.Boundaries("outer"))

f = LinearForm (fes)
f += v * g * ds("inner")
gfu = GridFunction (fes)

a.Assemble()
f.Assemble()


res = gfu.vec.CreateVector()
res.data = f.vec - a.mat * gfu.vec

gfu.vec.data += a.mat.Inverse(freedofs=fes.FreeDofs(), inverse="mumps") * res
output_path = os.path.dirname(os.path.realpath(__file__)) + "/output/"
writeVtk = 1
if (model=='full'):
    absvtk = 'fullmodel'
    
if (model=='half'):
    absvtk = 'halfmodel'    
if (writeVtk==1):
    if rank==0 and not os.path.exists(output_path):
        os.mkdir(output_path)
    comm.Barrier() 
    ff = ((gfu.real)**2 + (gfu.imag)**2)**0.5
    vtk = VTKOutput(ma=mesh, coefs=[ff], names=["uh"], filename=str(absvtk)+"_proc"+str(rank), subdivision=0)
    vtk.Do()
    print('Done')
 
